@@ -1,19 +1,18 @@
-#!/usr/bin/env perl
+use warnings;
 
 package Git::Hooks::CheckCommit;
 # ABSTRACT: Git::Hooks plugin to enforce commit policies
-$Git::Hooks::CheckCommit::VERSION = '2.0.0'; # TRIAL
-use 5.010;
+$Git::Hooks::CheckCommit::VERSION = '3.0.0'; # TRIAL
+use 5.016;
 use utf8;
-use strict;
-use warnings;
-use Try::Tiny;
+use Carp;
+use Log::Any '$log';
 use Git::Hooks;
 use Git::Repository::Log;
 use List::MoreUtils qw/any none/;
 
 my $PKG = __PACKAGE__;
-(my $CFG = __PACKAGE__) =~ s/.*::/githooks./;
+my $CFG = __PACKAGE__ =~ s/.*::/githooks./r;
 
 #############
 # Grok hook configuration, check it and set defaults.
@@ -25,11 +24,29 @@ sub _setup_config {
 
     $config->{lc $CFG} //= {};
 
+    my $default = $config->{lc $CFG};
+
+    $default->{'push-limit'} //= [0];
+
     return;
 }
 
 ##########
 
+# Return common help messages to fix author or committer name/email.  This
+# routine is used to compose some error messages in the routine match_errors
+# below. The "no critic" exemption below is a false positive.
+sub _amend_help {               ## no critic (ProhibitUnusedPrivateSubroutines)
+    my ($who) = @_;
+    if ($who eq 'author') {
+        return 'Please, amend your commit using the --author option to fix the author name/email.';
+    } elsif ($who eq 'committer') {
+        return 'Please, amend your commit after fixing your user.name and/or user.email configuration options.';
+    } else {
+        croak "Internal error: invalid who ($who)";
+    }
+}
+
 sub match_errors {
     my ($git, $commit) = @_;
 
@@ -54,13 +71,21 @@ sub match_errors {
                     my $who_info = "${who}_${info}";
                     my $data     = $commit->$who_info;
 
-                    unless (any  { $data =~ $_ } @{$checks->{''}}) {
-                        $git->error($PKG, "commit @{[$commit->commit]} $who $info ($data) does not match any positive githooks.checkcommit.$info option");
+                    if (none { $data =~ $_ } @{$checks->{''}}) {
+                        $git->fault(<<"EOS", {commit => $commit, option => $info});
+The commit $who $info ($data) is invalid.
+It must match at least one positive option.
+@{[_amend_help($who)]}
+EOS
                         ++$errors;
                     }
 
-                    unless (none { $data =~ $_ } @{$checks->{'!'}}) {
-                        $git->error($PKG, "commit @{[$commit->commit]} $who $info ($data) matches some negative githooks.checkcommit.$info option");
+                    if (any { $data =~ $_ } @{$checks->{'!'}}) {
+                        $git->fault(<<"EOS", {commit => $commit, option => $info});
+The commit $who $info ($data) is invalid.
+It matches some negative option.
+@{[_amend_help($who)]}
+EOS
                         ++$errors;
                     }
                 }
@@ -77,7 +102,14 @@ sub merge_errors {
     if ($commit->parent() > 1) { # it's a merge commit
         if (my @mergers = $git->get_config($CFG => 'merger')) {
             if (none {$git->match_user($_)} @mergers) {
-                $git->error($PKG, "commit @{[$commit->commit]} is a merge but you (@{[$git->authenticated_user]}) are not allowed to do merges");
+                $git->fault(<<"EOS", {commit => $commit, option => 'merger'});
+Authorization error: you cannot push this commit.
+
+I'm sorry, but you (@{[$git->authenticated_user]}) are not authorized to push
+*merge commits* because you're not included in the configuration option. You
+must either include yourself in that option or ask somebody else to push the
+commit for you.
+EOS
                 return 1;
             }
         }
@@ -93,20 +125,27 @@ sub email_valid_errors {
 
     my $cache = $git->cache($PKG);
 
-    if ($git->get_config($CFG => 'email-valid')) {
+    if ($git->get_config_boolean($CFG => 'email-valid')) {
         # Let's also cache the Email::Valid object
         unless (exists $cache->{email_valid}) {
             $cache->{email_valid} = undef;
             if (eval { require Email::Valid; }) {
                 my @checks;
                 foreach my $check (qw/mxcheck tldcheck fqdn allow_ip/) {
-                    if (my $value = $git->get_config($CFG => "email-valid.$check")) {
+                    if (my $value = $git->get_config_boolean($CFG => "email-valid.$check")) {
                         push @checks, "-$check" => $value;
                     }
                 }
                 $cache->{email_valid} = Email::Valid->new(@checks);
             } else {
-                $git->error($PKG, "the checkcommit.email-valid failed because the Email::Valid Perl module is not installed");
+                $git->fault(<<'EOS', {option => 'email-valid.*'});
+I could not load the Email::Valid Perl module.
+
+I need it to validate your commit's author and committer as requested by your
+configuration options.
+
+Please, install the module or disable the options to proceed.
+EOS
                 ++$errors;
             }
         }
@@ -116,8 +155,11 @@ sub email_valid_errors {
                 my $who_email = "${who}_email";
                 my $email     = $commit->$who_email;
                 unless ($ev->address($email)) {
-                    my $fail = $ev->details();
-                    $git->error($PKG, "commit @{[$commit->commit]} $who email ($email) failed $fail check");
+                    my $details = $ev->details();
+                    $git->fault(<<"EOS", {commit => $commit});
+The commit $who email ($email) failed the $details check.
+@{[_amend_help($who)]}
+EOS
                     ++$errors;
                 }
             }
@@ -134,21 +176,7 @@ sub _canonical_identity {
 
     unless (exists $cache->{canonical}{$identity}) {
         $cache->{canonical}{$identity} =
-            try {
-                chomp(my $canonical = $git->run(
-                    '-c', "mailmap.file=$mailmap",
-                    'check-mailmap',
-                    $identity,
-                ));
-                $canonical;
-            } catch {
-                $git->error($PKG, <<'EOS');
-The githooks.checkcommit.canonical option requires the git-check-mailmap
-command which isn't found. It's available since Git 1.8.4. You should either
-upgrade your Git or disable this option.
-EOS
-                $identity;
-        };
+            $git->run('-c', "mailmap.file=$mailmap", 'check-mailmap', $identity);
     }
 
     return $cache->{canonical}{$identity};
@@ -164,13 +192,23 @@ sub canonical_errors {
             my $who_name  = "${who}_name";
             my $who_email = "${who}_email";
             my $identity  = $commit->$who_name . ' <' . $commit->$who_email . '>';
-            my $canonical = _canonical_identity($git, $mailmap, $identity);
+            my $canonical = eval {_canonical_identity($git, $mailmap, $identity) };
+            unless (defined $canonical) {
+                $git->fault(<<'EOS', {option => 'canonical'});
+Git error: could not run command git-check-mailmap.
+The configuration option requires it.
+It's available since Git 1.8.4.
+Please, either upgrade your Git or disable this option.
+EOS
+                ++$errors;
+            }
 
             if ($identity ne $canonical) {
-                $git->error(
-                    $PKG,
-                    "commit @{[$commit->commit]} $who identity ($identity) isn't canonical ($canonical)",
-                );
+                $git->fault(<<"EOS", {commit => $commit, option => 'canonical'});
+The commit $who identity isn't canonical.
+It's '$identity' but its canonical form is '$canonical'.
+@{[_amend_help($who)]}
+EOS
                 ++$errors;
             }
         }
@@ -187,16 +225,25 @@ sub signature_errors {
     my $signature = $git->get_config($CFG => 'signature');
 
     if (defined $signature && $signature ne 'nocheck') {
-        my $status = $git->run(qw/log -1 --format='%G?'/, $commit->commit);
+        my $status = $git->run(qw/log -1 --format=%G?/, $commit);
 
         if ($status eq 'B') {
-            $git->error($PKG, "commit @{[$commit->commit]} has a BAD signature");
+            $git->fault(<<'EOS', {commit => $commit, option => 'signature'});
+The commit has a BAD GPG signature.
+Please, amend your commit with the -S option to fix it.
+EOS
             ++$errors;
-        } elsif ($signature ne 'optional' && $status eq 'N') {
-            $git->error($PKG, "commit @{[$commit->commit]} has NO signature");
+        } elsif ($status eq 'N' && $signature ne 'optional') {
+            $git->fault(<<'EOS', {commit => $commit, option => 'signature'});
+The commit has NO GPG signature.
+Please, amend your commit with the -S option to add one.
+EOS
             ++$errors;
-        } elsif ($signature eq 'trusted' && $status eq 'U') {
-            $git->error($PKG, "commit @{[$commit->commit]} has an UNTRUSTED signature");
+        } elsif ($status eq 'U' && $signature eq 'trusted') {
+            $git->fault(<<'EOS', {commit => $commit, option => 'signature'});
+The commit has an UNTRUSTED GPG signature.
+Please, amend your commit with the -S option to add another one.
+EOS
             ++$errors;
         }
     }
@@ -209,10 +256,10 @@ sub code_errors {
 
     my $errors = 0;
 
-    state $codes;
+    my $cache = $git->cache($PKG);
 
-    unless (ref $codes) {
-        $codes = [];
+    unless (exists $cache->{codes}) {
+        $cache->{codes} = [];
       CODE:
         foreach my $check ($git->get_config($CFG => 'check-code')) {
             my $code;
@@ -220,11 +267,14 @@ sub code_errors {
                 $code = do $check;
                 unless ($code) {
                     if (length $@) {
-                        $git->error($PKG, "couldn't parse check-code file ($check): ", $@);
+                        $git->fault("I couldn't parse the file ($check):",
+                                    {commit => $commit, option => 'check-code', details => $@});
                     } elsif (! defined $code) {
-                        $git->error($PKG, "couldn't read check-code file ($check): ", $!);
+                        $git->fault("I couldn't read the file ($check):",
+                                    {commit => $commit, option => 'check-code', details => $@});
                     } else {
-                        $git->error($PKG, "check-code file ($check) returned FALSE");
+                        $git->fault("The file ($check) returned FALSE",
+                                    {commit => $commit, option => 'check-code', details => $@});
                     }
                     ++$errors;
                     next CODE;
@@ -232,29 +282,33 @@ sub code_errors {
             } else {
                 $code = eval $check; ## no critic (BuiltinFunctions::ProhibitStringyEval)
                 if (length $@) {
-                    $git->error($PKG, "couldn't parse check-code value: ", $@);
+                    $git->fault("I couldn't parse the option value ($check):",
+                                {commit => $commit, option => 'check-code', details => $@});
                     ++$errors;
                     next CODE;
                 }
             }
             if (defined $code && ref $code && ref $code eq 'CODE') {
-                push @$codes, $code;
+                push @{$cache->{codes}}, $code;
             } else {
-                $git->error($PKG, "option check-code must end with a code ref");
+                $git->fault("The option value must end with a code ref.",
+                            {commit => $commit, option => 'check-code'});
                 ++$errors;
             }
         }
     }
 
-    foreach my $code (@$codes) {
+    foreach my $code (@{$cache->{codes}}) {
         my $ok = eval { $code->($git, $commit, $ref) };
         if (defined $ok) {
             unless ($ok) {
-                $git->error($PKG, "error while evaluating check-code");
+                $git->fault("Error detected while evaluating the option.",
+                            {commit => $commit, option => 'check-code'});
                 ++$errors;
             }
         } elsif (length $@) {
-            $git->error($PKG, 'error while evaluating check-code', $@);
+            $git->fault('Error detected while evaluating the option',
+                        {commit => $commit, option => 'check-code', details => $@});
             ++$errors;
         }
     }
@@ -281,9 +335,18 @@ sub check_ref {
 
     my @commits = $git->get_affected_ref_commits($ref);
 
-    if (my $limit = $git->get_config($CFG => 'push-limit')) {
+    if (my $limit = $git->get_config_integer($CFG => 'push-limit')) {
         if (@commits > $limit) {
-            $git->error($PKG, "you're pushing @{[scalar @commits]} commits to $ref, more than our current limit of $limit");
+            $git->fault(<<"EOS", {ref => $ref, option => 'push-limit'});
+Are you sure you want to push @{[scalar @commits]} commits to this reference at
+once?
+
+The configuration option currently allows one to push at most $limit commits to
+a reference at once.
+
+If you're sure about this you can break the whole commit sequence in smaller
+subsequences and push them one at a time.
+EOS
             ++$errors;
         }
     }
@@ -295,85 +358,63 @@ sub check_ref {
     return $errors;
 }
 
-sub check_pre_commit {
+sub check_commit {
     my ($git) = @_;
 
-    _setup_config($git);
+    # Grok author and committer information from git's environment variables, if
+    # they're defined. Sometimes they aren't...
+
+    my $author_name     = $ENV{GIT_AUTHOR_NAME}     || 'nobody';
+    my $author_email    = $ENV{GIT_AUTHOR_EMAIL}    || 'nobody@example.net';
+    my $committer_name  = $ENV{GIT_COMMITTER_NAME}  || $author_name;
+    my $committer_email = $ENV{GIT_COMMITTER_EMAIL} || $author_email;
 
     # Construct a fake commit object to pass to the error checking routines.
     my $commit = Git::Repository::Log->new(
         commit    => '<new>',
-        author    => "$ENV{GIT_AUTHOR_NAME} <$ENV{GIT_AUTHOR_EMAIL}> 1234567890 -0300",
-        committer => "$ENV{GIT_COMMITTER_NAME} <$ENV{GIT_COMMITTER_EMAIL}> 1234567890 -0300",
+        author    => "$author_name <$author_email> 1234567890 -0300",
+        committer => "$committer_name <$committer_email> 1234567890 -0300",
         message   => "Fake\n",
     );
 
-    return 0 ==
-        (match_errors($git, $commit) +
-         email_valid_errors($git, $commit) +
-         canonical_errors($git, $commit) +
-         code_errors($git, $commit));
+    return
+        match_errors($git, $commit) +
+        email_valid_errors($git, $commit) +
+        canonical_errors($git, $commit) +
+        code_errors($git, $commit);
 }
 
 sub check_post_commit {
     my ($git) = @_;
 
+    $log->debug(__PACKAGE__ . "::check_post_commit");
+
     _setup_config($git);
 
+    my $current_branch = $git->get_current_branch();
+
+    return 1 unless $git->is_reference_enabled($current_branch);
+
     my $commit = $git->get_sha1('HEAD');
 
-    if (signature_errors($git, $commit)) {
-        $git->error($PKG, "broken commit", <<"EOF");
-ATTENTION: To fix the problems in this commit, please consider
-amending it:
-
-        git commit --amend      # to amend it
-EOF
-        return 0;
-    } else {
-        return 1;
-    }
-
-}
-
-# This routine can act both as an update or a pre-receive hook.
-sub check_affected_refs {
-    my ($git) = @_;
-
-    _setup_config($git);
-
-    return 1 if $git->im_admin();
-
-    my $errors = 0;
-
-    foreach my $ref ($git->get_affected_refs()) {
-        $errors += check_ref($git, $ref);
-    }
-
-    return $errors == 0;
+    return signature_errors($git, $commit);
 }
 
 sub check_patchset {
-    my ($git, $opts) = @_;
+    my ($git, $branch, $commit) = @_;
 
-    _setup_config($git);
-
-    return 1 if $git->im_admin();
-
-    my $sha1   = $opts->{'--commit'};
-    my $commit = $git->get_commit($sha1);
-
-    return commit_errors($git, $commit) == 0;
+    return commit_errors($git, $commit, $branch);
 }
 
 # Install hooks
-PRE_COMMIT       \&check_pre_commit;
+my $options = {config => \&_setup_config};
+
+GITHOOKS_CHECK_AFFECTED_REFS \&check_ref,      $options;
+GITHOOKS_CHECK_PRE_COMMIT    \&check_commit,   $options;
+GITHOOKS_CHECK_PATCHSET      \&check_patchset, $options;
+
+POST_APPLYPATCH  \&check_post_commit;
 POST_COMMIT      \&check_post_commit;
-UPDATE           \&check_affected_refs;
-PRE_RECEIVE      \&check_affected_refs;
-REF_UPDATE       \&check_affected_refs;
-PATCHSET_CREATED \&check_patchset;
-DRAFT_PUBLISHED  \&check_patchset;
 
 1;
 
@@ -389,21 +430,58 @@ Git::Hooks::CheckCommit - Git::Hooks plugin to enforce commit policies
 
 =head1 VERSION
 
-version 2.0.0
+version 3.0.0
+
+=head1 SYNOPSIS
+
+As a C<Git::Hooks> plugin you don't use this Perl module directly. Instead, you
+may configure it in a Git configuration file like this:
+
+  [githooks]
+
+    # Enable the plugin
+    plugin = CheckCommit
+
+    # These users are exempt from all checks
+    admin = joe molly
+
+    # The @mergers group is used below
+    groups = mergers = larry sally
+
+  [githooks "checkcommit"]
+
+    # Reject commits if the author or committer name contains any characters
+    # other then lowercase letters.
+    name = !^[a-z]+$
+
+    # Reject commits if the author or committer email does not belong to the
+    # @cpqd.com.br domain.
+    email = @cpqd\.com\.br$
+
+    # Enable several integrity checks on the author and committer emails using
+    # the Email::Valid Perl module.
+    email-valid = true
+
+    # Only users in the @mergers group can push merge commits.
+    merger = @mergers
+
+    # Rejects pushes with more than two commits in a single branch, in order to
+    # avoid careless pushes.
+    push-limit = 2
 
 =head1 DESCRIPTION
 
-This Git::Hooks plugin hooks itself to the hooks below to enforce
-commit policies.
+This L<Git::Hooks> plugin hooks itself to the hooks below to enforce commit
+policies.
 
 =over
 
-=item * B<pre-commit>
+=item * B<pre-commit>, B<pre-applypatch>
 
 This hook is invoked before a commit is made to check the author and
 committer identities.
 
-=item * B<post-commit>
+=item * B<post-commit>, B<post-applypatch>
 
 This hook is invoked after a commit is made to check its signature. Note
 that the commit is checked after is has been made and any errors must be
@@ -422,8 +500,18 @@ check if all commits being pushed comply.
 
 =item * B<ref-update>
 
-This hook is invoked when a push request is received by Gerrit Code Review,
-to check if all commits being pushed comply.
+This hook is invoked when a direct push request is received by Gerrit Code
+Review, to check if all commits being pushed comply.
+
+=item * B<commit-received>
+
+This hook is invoked when a push request is received by Gerrit Code Review to
+create a change for review, to check if all commits being pushed comply.
+
+=item * B<submit>
+
+This hook is invoked when a change is submitted in Gerrit Code Review, to check
+if all commits being pushed comply.
 
 =item * B<patchset-created>
 
@@ -441,19 +529,25 @@ commit signatures.
 To enable this plugin you should add it to the githooks.plugin configuration
 option:
 
-    git config --add githooks.plugin CheckCommit
+    [githooks]
+      plugin = CheckCommit
 
-=for Pod::Coverage match_errors merge_errors email_valid_errors canonical_errors identity_errors signature_errors spelling_errors pattern_errors subject_errors body_errors footer_errors commit_errors code_errors check_pre_commit check_post_commit check_ref check_affected_refs check_patchset
+=for Pod::Coverage match_errors merge_errors email_valid_errors canonical_errors identity_errors signature_errors spelling_errors pattern_errors subject_errors body_errors footer_errors commit_errors code_errors check_commit check_post_commit check_ref check_patchset
 
 =head1 NAME
 
-Git::Hooks::CheckCommit - Git::Hooks plugin to enforce commit policies.
+Git::Hooks::CheckCommit - Git::Hooks plugin to enforce commit policies
 
 =head1 CONFIGURATION
 
-The plugin is configured by the following git options.
+The plugin is configured by the following git options under the
+C<githooks.checkcommit> subsection.
 
-=head2 githooks.checkcommit.name [!]REGEXP
+It can be disabled for specific references via the C<githooks.ref> and
+C<githooks.noref> options about which you can read in the L<Git::Hooks>
+documentation.
+
+=head2 name [!]REGEXP
 
 This multi-valued option impose restrictions on the valid author and
 committer names using regular expressions.
@@ -464,7 +558,13 @@ regular expressions (the ones prefixed by "!").
 
 This check is performed by the C<pre-commit> local hook.
 
-=head2 githooks.checkcommit.email [!]REGEXP
+This allows you, for example, to require that author and committer names have at
+least a first and a last name, separated by spaces:
+
+  [githooks "checklog"]
+    name = .\\s+.
+
+=head2 email [!]REGEXP
 
 This multi-valued option impose restrictions on the valid author and
 committer emails using regular expressions.
@@ -475,7 +575,7 @@ negative regular expressions (the ones prefixed by "!").
 
 This check is performed by the C<pre-commit> local hook.
 
-=head2 githooks.checkcommit.email-valid [01]
+=head2 email-valid BOOL
 
 This option uses the L<Email::Valid> module' C<address> method to validate
 author and committer email addresses.
@@ -487,31 +587,31 @@ L<Git::Hooks>.  If it's not found or if there's an error in the construction
 of the C<Email::Valid> object the check fails with a suitable message.
 
 The C<Email::Valid> constructor (new) accepts some parameters. You can pass
-the boolean parameters to change their default values by means of the
+the Boolean parameters to change their default values by means of the
 following sub-options. For more information, please consult the
 L<Email::Valid> documentation.
 
-=head3 githooks.checkcommit.email-valid.mxcheck [01]
+=head3 githooks.checkcommit.email-valid.mxcheck BOOL
 
 Specifies whether addresses should be checked for a valid DNS entry. The
 default is false.
 
-=head3 githooks.checkcommit.email-valid.tldcheck [01]
+=head3 githooks.checkcommit.email-valid.tldcheck BOOL
 
 Specifies whether addresses should be checked for valid top level
 domains. The default is false.
 
-=head3 githooks.checkcommit.email-valid.fqdn [01]
+=head3 githooks.checkcommit.email-valid.fqdn BOOL
 
 Species whether addresses must contain a fully qualified domain name
 (FQDN). The default is true.
 
-=head3 githooks.checkcommit.email-valid.allow_ip [01]
+=head3 githooks.checkcommit.email-valid.allow_ip BOOL
 
 Specifies whether a "domain literal" is acceptable as the domain part.  That
 means addresses like: C<rjbs@[1.2.3.4]>. The default is true.
 
-=head2 githooks.checkcommit.canonical MAILMAP
+=head2 canonical MAILMAP
 
 This option requires the use of canonical names and emails for authors and
 committers, as configured in a F<MAILMAP> file and checked by the
@@ -537,7 +637,7 @@ to temporarily configure it to use the F<MAILMAP> file.
 
 These checks are performed by the C<pre-commit> local hook.
 
-=head2 githooks.checkcommit.signature {nocheck|optional|good|trusted}
+=head2 signature {nocheck|optional|good|trusted}
 
 This option allows one to check commit signatures according to these values:
 
@@ -566,14 +666,14 @@ signatures.
 
 This check is performed by the C<post-commit> local hook.
 
-=head2 githooks.checkcommit.merger WHO
+=head2 merger WHO
 
 This multi-valued option restricts who can push commit merges to the
 repository. WHO may be specified as a username, a groupname, or a regex,
 like the C<githooks.admin> option (see L<Git::Hooks/CONFIGURATION>) so that
 only users matching WHO may push merge commits.
 
-=head2 githooks.checkcommit.push-limit N
+=head2 push-limit INT
 
 This limits the number of commits that may be pushed at once on top of any
 reference. Set it to 1 to force developers to squash their commits before
@@ -581,7 +681,7 @@ pushing them. Or set it to a low number (such as 3) to deny long chains of
 commits to be pushed, which are usually made by Git newbies who don't know
 yet how to amend commits. ;-)
 
-=head2 githooks.checkcommit.check-code CODESPEC
+=head2 check-code CODESPEC
 
 If the above checks aren't enough you can use this option to define a custom
 code to check your commits. The code may be specified directly as the
@@ -610,7 +710,7 @@ B<pre-receive> hooks. For the B<pre-commit> hook this argument is B<undef>.
 
 =back
 
-The subroutine should return a boolean value indicating success. Any errors
+The subroutine should return a Boolean value indicating success. Any errors
 should be produced by invoking the
 B<Git::Repository::Plugin::GitHooks::error> method.
 
@@ -637,7 +737,7 @@ Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by CPqD <www.cpqd.com.br>.
+This software is copyright (c) 2020 by CPQD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
