@@ -1,20 +1,18 @@
-#!/usr/bin/env perl
+use warnings;
 
 package Git::Hooks::CheckJira;
 # ABSTRACT: Git::Hooks plugin which requires citation of JIRA issues in commit messages
-$Git::Hooks::CheckJira::VERSION = '2.0.0'; # TRIAL
-use 5.010;
+$Git::Hooks::CheckJira::VERSION = '3.0.0'; # TRIAL
+use 5.016;
 use utf8;
-use strict;
-use warnings;
+use Log::Any '$log';
 use Git::Hooks;
 use Git::Repository::Log;
 use Path::Tiny;
-use List::MoreUtils qw/uniq/;
-use Set::Scalar;
+use List::MoreUtils qw/last_index uniq/;
 
 my $PKG = __PACKAGE__;
-(my $CFG = __PACKAGE__) =~ s/.*::/githooks./;
+my $CFG = __PACKAGE__ =~ s/.*::/githooks./r;
 
 #############
 # Grok hook configuration, check it and set defaults.
@@ -31,8 +29,8 @@ sub _setup_config {
     # Default matchkey for matching default JIRA keys.
     $default->{matchkey}   //= ['\b[A-Z][A-Z]+-\d+\b'];
 
-    $default->{require}    //= [1];
-    $default->{unresolved} //= [1];
+    $default->{require}    //= ['true'];
+    $default->{unresolved} //= ['true'];
 
     return;
 }
@@ -67,21 +65,37 @@ sub _jira {
     # Connect to JIRA if not yet connected
     unless (exists $cache->{jira}) {
         unless (eval { require JIRA::REST; }) {
-            $git->error($PKG, "Please, install Perl module JIRA::REST to use the CheckJira plugin", $@);
+            $git->fault(<<"EOS", {details => $@});
+I could not load the JIRA::REST Perl module.
+
+I need it to talk to your JIRA server, as configured by the
+$CFG.jiraurl, $CFG.jirauser, and $CFG.jirapass
+options in your Git configuration.
+
+Please, install the module or disable these options to proceed.
+EOS
             return;
         }
 
         my %jira;
         for my $option (qw/jiraurl jirauser jirapass/) {
             $jira{$option} = $git->get_config($CFG => $option)
-                or $git->error($PKG, "missing $CFG.$option configuration attribute")
-                    and return;
+                or $git->fault(<<'EOS', {option => $option})
+The option is missing from the configuration.
+It's required in order to connect to the JIRA server.
+EOS
+                and return;
         }
         $jira{jiraurl} =~ s:/+$::; # trim trailing slashes from the URL
 
         my $jira = eval { JIRA::REST->new($jira{jiraurl}, $jira{jirauser}, $jira{jirapass}) };
         length $@
-            and $git->error($PKG, "cannot connect to the JIRA server at '$jira{jiraurl}' as '$jira{jirauser}", $@)
+            and $git->fault(<<"EOS", {details => $@})
+Cannot connect to the JIRA server at '$jira{jiraurl}' as '$jira{jirauser}.
+
+Please, check your $CFG.jiraurl, $CFG.jirauser,
+and $CFG.jirapass configuration options.
+EOS
                 and return;
         $cache->{jira} = $jira;
     }
@@ -119,44 +133,56 @@ sub _disconnect_jira {
 sub check_codes {
     my ($git) = @_;
 
-    my @codes;
+    my $cache = $git->cache($PKG);
 
-  CODE:
-    foreach my $check ($git->get_config($CFG => 'check-code')) {
-        my $code;
-        if ($check =~ s/^file://) {
-            $code = do $check;
-            unless ($code) {
-                if (length $@) {
-                    $git->error($PKG, "couldn't parse option check-code ($check)", $@);
-                } elsif (! defined $code) {
-                    $git->error($PKG, "couldn't do option check-code ($check)", $!);
-                } else {
-                    $git->error($PKG, "couldn't run option check-code ($check)");
+    unless (exists $cache->{codes}) {
+        $cache->{codes} = [];
+      CODE:
+        foreach my $check ($git->get_config($CFG => 'check-code')) {
+            my $code;
+            if ($check =~ s/^file://) {
+                $code = do $check;
+                unless ($code) {
+                    if (length $@) {
+                        $git->fault("I couldn't parse option value ($check).",
+                                    {option => 'check-code', details => $@});
+                    } elsif (! defined $code) {
+                        $git->fault("I couldn't do option value ($check).",
+                                    {option => 'check-code', details => $!});
+                    } else {
+                        $git->fault("I couldn't run  option value ($check).",
+                                    {option => 'check-code'});
+                    }
+                    next CODE;
                 }
-                next CODE;
-            }
-        } else {
-            $code = eval $check; ## no critic (BuiltinFunctions::ProhibitStringyEval)
-            length $@
-                and $git->error($PKG, "couldn't parse option check-code value", $@)
+            } else {
+                $code = eval $check; ## no critic (BuiltinFunctions::ProhibitStringyEval)
+                length $@
+                    and $git->fault("I couldn't parse option value.",
+                                    {option => 'check-code', details => $@})
                     and next CODE;
-        }
-        defined $code and ref $code and ref $code eq 'CODE'
-            or $git->error($PKG, "option check-code must end with a code ref")
+            }
+            defined $code and ref $code and ref $code eq 'CODE'
+                or $git->fault("The option value must end with a code-ref.",
+                               {option => 'check-code'})
                 and next CODE;
-        push @codes, $code;
+            push @{$cache->{codes}}, $code;
+        }
     }
 
-    return @codes;
+    return @{$cache->{codes}};
 }
 
 sub _check_jira_keys {          ## no critic (ProhibitExcessComplexity)
     my ($git, $commit, $ref, @keys) = @_;
 
     unless (@keys) {
-        if ($git->get_config($CFG => 'require')) {
-            $git->error($PKG, "commit @{[substr($commit->commit, 0, 10)]} must cite a JIRA in its message");
+        if ($git->get_config_boolean($CFG => 'require')) {
+            $git->fault(<<'EOS', {commit => $commit});
+The commit must cite a JIRA in its message.
+
+Please, amend your commit to insert a JIRA key.
+EOS
             return 0;
         } else {
             return 1;
@@ -171,55 +197,68 @@ sub _check_jira_keys {          ## no critic (ProhibitExcessComplexity)
     # JQL checks
 
     {
-        # global JQLs
-        my @jqls = $git->get_config($CFG => 'jql');
+        # Build a list of JQL terms
 
-        # ref-specific JQLs
-        foreach ($git->get_config($CFG => 'ref-jql')) {
-            my ($match_ref, $jql) = split ' ', $_, 2;
-            push @jqls, $jql if $ref =~ $match_ref;
+        # Starting with a check to see if all JIRA keys exist
+        my @jqls = ("key IN (@{[join(',', @keys)]})");
+
+        # global JQL
+        if (my $jql = $git->get_config($CFG => 'jql')) {
+            push @jqls, $jql unless $jql eq 'undef';
         }
 
-        # Construct a JQL based on the deprecated configuration options
-        {
-            my @deprecated;
-            foreach my $option (qw/project issuetype status/) {
-                if (my @values = $git->get_config($CFG => $option)) {
-                    push @deprecated, "$option IN (@{[join(',', @values)]})";
-                }
+        # ref-specific JQL
+        foreach my $refjql (reverse $git->get_config($CFG => 'ref-jql')) {
+            my ($match_ref, $jql) = split ' ', $refjql, 2;
+            if ($ref =~ $match_ref) {
+                push @jqls, $jql unless $jql eq 'undef';
+                last;
             }
-            push @jqls, join(' AND ', @deprecated) if @deprecated;
         }
 
-        my $match_keys = "key IN (@{[join(',', @keys)]})";
-        my $cited_set  = Set::Scalar->new(@keys);
+        # AND JQLs
+        if (my @and_jql = $git->get_config($CFG => 'and-jql')) {
+            my $first = 1 + last_index {$_ eq 'undef'} @and_jql;
+            push @jqls, @and_jql[$first .. $#and_jql];
+        }
 
-        # Conjunct $match_keys with each JQL expression to match only the
-        # cited issues.
-        @jqls = map {"$match_keys AND $_"} @jqls;
+        # Conjunct all terms in a single JQL expression
+        my $JQL = '(' . join(') AND (', @jqls) . ')';
 
-        # If there is no JQL expression used to restrict the search we
-        # inject an expression to search for the cited issues.
-        @jqls = ($match_keys) unless @jqls;
+        # Squeeze multiple whitespaces in a single space to make it appear
+        # neatly in error messages.
+        $JQL =~ s/\s{2,}/ /g;
 
-        foreach my $jql (@jqls) {
-            my $issues = _jql_query($git, $jql);
+        my $issues = _jql_query($git, $JQL);
 
-            @issues{keys %$issues} = values %$issues; # cache all matched issues
+        @issues{keys %$issues} = values %$issues; # cache all matched issues
 
-            my $matched_set = Set::Scalar->new(keys %$issues);
-            if (my $missed_set = $cited_set - $matched_set) {
-                $git->error($PKG, "the JIRA issue(s) @{[$missed_set]} do(es) not match the expression '$jql'");
-                ++$errors;
-            }
+        if (my @issues_not_found  = sort grep { ! exists $issues->{$_} } @keys) {
+            # Some issue keys were cited but not found in JIRA
+            ++$errors;
+            local $, = ' ';
+            $git->fault(<<"EOS", {commit => $commit});
+The commit cites the following invalid issues:
+
+  @issues_not_found
+
+The issues do not match the following JQL expression:
+
+  $JQL
+
+Please, update your issues or fix your $CFG git configuration.
+EOS
         }
     }
 
+    # Return prematurely if there are no issues to check
+    return $errors == 0 unless %issues;
+
     ################
     # Non-JQL checks
     {
-        my $unresolved  = $git->get_config($CFG => 'unresolved');
-        my $by_assignee = $git->get_config($CFG => 'by-assignee');
+        my $unresolved  = $git->get_config_boolean($CFG => 'unresolved');
+        my $by_assignee = $git->get_config_boolean($CFG => 'by-assignee');
         my @versions;
         foreach ($git->get_config($CFG => 'fixversion')) {
             my ($branch, $version) = split ' ', $_, 2;
@@ -242,7 +281,11 @@ sub _check_jira_keys {          ## no critic (ProhibitExcessComplexity)
       ISSUE:
         while (my ($key, $issue) = each %issues) {
             if ($unresolved && defined $issue->{fields}{resolution}) {
-                $git->error($PKG, "issue $key cannot be used because it is already resolved");
+                $git->fault(<<"EOS", {commit => $commit, option => 'unresolved'});
+The commit cites issue $key which is already resolved.
+
+The option in your configuration requires that all JIRA issues be unresolved.
+EOS
                 ++$errors;
                 next ISSUE;
             }
@@ -256,25 +299,42 @@ sub _check_jira_keys {          ## no critic (ProhibitExcessComplexity)
                         next VERSION if $fixversion->{name} eq $version;
                     }
                 }
-                $git->error($PKG, "issue $key has no fixVersion matching '$version', which is required for commits affecting '$ref'");
+                $git->fault(<<"EOS", {commit => $commit, option => 'fixversion'});
+The commit cites issue $key which is invalid.
+
+Commits on '$ref' must cite issues associated with a fixVersion matching
+'$version' according to the configuration option.
+EOS
                 ++$errors;
                 next ISSUE;
             }
 
             if ($by_assignee) {
                 my $user = $git->authenticated_user()
-                    or $git->error($PKG, "cannot grok the authenticated user")
+                    or $git->fault(<<'EOS')
+Internal error: I cannot get your username to authorize you.
+Please check your Git::Hooks configuration with regards to the function
+https://metacpan.org/pod/Git::Repository::Plugin::GitHooks#authenticated_user
+EOS
                     and ++$errors
                     and next ISSUE;
 
                 if (my $assignee = $issue->{fields}{assignee}) {
                     my $name = $assignee->{name};
                     $user eq $name
-                        or $git->error($PKG, "issue $key should be assigned to '$user', not '$name'")
+                        or $git->fault(<<"EOS", {commit => $commit, option => 'by-assignee'})
+The commit cites issue $key which is assigned to '$name'.
+The option requires that cited issues be assigned to you ($user).
+Please, update your issue.
+EOS
                         and ++$errors
                         and next KEY;
                 } else {
-                    $git->error($PKG, "issue $key should be assigned to '$user', but it's unassigned");
+                    $git->fault(<<"EOS", {commit => $commit, option => 'by-assignee'});
+The commit cites issue $key which is unassigned.
+The option requires that cited issues be assigned to you ($user).
+Please, update your issue.
+EOS
                     ++$errors;
                     next KEY;
                 }
@@ -291,7 +351,8 @@ sub _check_jira_keys {          ## no critic (ProhibitExcessComplexity)
             if (defined $ok) {
                 ++$errors unless $ok;
             } elsif (length $@) {
-                $git->error($PKG, 'error while evaluating check-code', $@);
+                $git->fault('Error while evaluating option value.',
+                            {option => 'check-code', details => $@});
                 ++$errors;
             }
         } else {
@@ -305,7 +366,7 @@ sub _check_jira_keys {          ## no critic (ProhibitExcessComplexity)
 sub check_commit_msg {
     my ($git, $commit, $ref) = @_;
 
-    if ($commit->parent() > 1 && $git->get_config($CFG => 'skip-merges')) {
+    if ($commit->parent() > 1 && $git->get_config_boolean($CFG => 'skip-merges')) {
         return 1;
     } else {
         return _check_jira_keys($git, $commit, $ref, uniq(grok_msg_jiras($git, $commit->message)));
@@ -313,39 +374,13 @@ sub check_commit_msg {
 }
 
 sub check_patchset {
-    my ($git, $opts) = @_;
+    my ($git, $branch, $commit) = @_;
 
-    _setup_config($git);
-
-    return 1 if $git->im_admin();
-
-    my $sha1   = $opts->{'--commit'};
-    my $commit = $git->get_commit($sha1);
-
-    # The --branch argument contains the branch short-name if it's in the
-    # refs/heads/ namespace. But we need to always use the branch long-name,
-    # so we change it here.
-    my $branch = $opts->{'--branch'};
-    $branch = "refs/heads/$branch"
-        unless $branch =~ m:^refs/:;
-
-    return 1 unless $git->is_ref_enabled($branch, $git->get_config($CFG => 'ref'));
-
-    return check_commit_msg($git, $commit, $branch);
+    return check_commit_msg($git, $commit, $branch) ? 0 : 1;
 }
 
 sub check_message_file {
-    my ($git, $commit_msg_file) = @_;
-
-    _setup_config($git);
-
-    my $current_branch = $git->get_current_branch();
-    return 1 unless $git->is_ref_enabled($current_branch, $git->get_config($CFG => 'ref'));
-
-    my $msg = eval { path($commit_msg_file)->slurp };
-    defined $msg
-        or $git->error($PKG, "cannot open file '$commit_msg_file' for reading: $@")
-            and return 0;
+    my ($git, $msg, $current_branch) = @_;
 
     # Remove comment lines from the message file contents.
     $msg =~ s/^#[^\n]*\n//mgs;
@@ -358,14 +393,12 @@ sub check_message_file {
         message   => $msg,
     );
 
-    return check_commit_msg($git, $commit, $current_branch);
+    return check_commit_msg($git, $commit, $current_branch) ? 0 : 1;
 }
 
 sub check_ref {
     my ($git, $ref) = @_;
 
-    return 1 unless $git->is_ref_enabled($ref, $git->get_config($CFG => 'ref'));
-
     my $errors = 0;
 
     foreach my $commit ($git->get_affected_ref_commits($ref)) {
@@ -378,26 +411,6 @@ sub check_ref {
     return $errors == 0;
 }
 
-# This routine can act both as an update or a pre-receive hook.
-sub check_affected_refs {
-    my ($git) = @_;
-
-    _setup_config($git);
-
-    return 1 if $git->im_admin();
-
-    my $errors = 0;
-
-    foreach my $ref ($git->get_affected_refs()) {
-        check_ref($git, $ref)
-            or ++$errors;
-    }
-
-    _disconnect_jira($git);
-
-    return $errors == 0;
-}
-
 sub notify_commit_msg {
     my ($git, $commit, $ref, $visibility) = @_;
 
@@ -410,13 +423,13 @@ sub notify_commit_msg {
     my $show = $git->run(show => '--stat', $commit->commit);
 
     my %comment = (
-        body => <<"EOF",
+        body => <<"EOS",
 [$PKG] commit refers to this issue:
 
 {noformat}
 $show
 {noformat}
-EOF
+EOS
     );
     $comment{visibility} = $visibility if $visibility;
 
@@ -424,7 +437,8 @@ EOF
 
     foreach my $key (@keys) {
         eval { $jira->POST("/issue/$key/comment", undef, \%comment); 1; }
-            or $git->error($PKG, "Cannot add a comment to JIRA issue $key:", $@)
+            or $git->fault("I could not add a comment to JIRA issue $key.",
+                           {commit => $commit, details => $@})
             and ++$errors;
     }
 
@@ -434,8 +448,6 @@ EOF
 sub notify_ref {
     my ($git, $ref, $visibility) = @_;
 
-    return 1 unless $git->is_ref_enabled($ref, $git->get_config($CFG => 'ref'));
-
     my $errors = 0;
 
     foreach my $commit ($git->get_affected_ref_commits($ref)) {
@@ -449,6 +461,8 @@ sub notify_ref {
 sub notify_affected_refs {
     my ($git) = @_;
 
+    $log->debug(__PACKAGE__ . "::notify_affected_refs");
+
     _setup_config($git);
 
     my $comment = $git->get_config($CFG => 'comment');
@@ -462,13 +476,20 @@ sub notify_affected_refs {
             value => $2,
         };
     } elsif ($comment ne 'all') {
-        $git->error($PKG, "Invalid argument to githooks.checkjira.comment: $comment");
+        $git->fault(<<"EOS", {option => $comment});
+Configuration error.
+
+The option is defined as '$comment', but
+the valid values are 'role:ROLE', 'group:GROUP', or 'all'.
+Please, check your git configuration.
+EOS
         return 0;
     }
 
     my $errors = 0;
 
     foreach my $ref ($git->get_affected_refs()) {
+        next unless $git->is_reference_enabled($ref);
         $errors += notify_ref($git, $ref, $visibility);
     }
 
@@ -478,13 +499,17 @@ sub notify_affected_refs {
 }
 
 # Install hooks
-COMMIT_MSG       \&check_message_file;
-UPDATE           \&check_affected_refs;
-PRE_RECEIVE      \&check_affected_refs;
-REF_UPDATE       \&check_affected_refs;
-POST_RECEIVE     \&notify_affected_refs;
-PATCHSET_CREATED \&check_patchset;
-DRAFT_PUBLISHED  \&check_patchset;
+my $options = {
+    config  => \&_setup_config,
+    destroy => \&_disconnect_jira,
+};
+
+GITHOOKS_CHECK_AFFECTED_REFS \&check_ref,          $options;
+GITHOOKS_CHECK_PATCHSET      \&check_patchset,     $options;
+GITHOOKS_CHECK_MESSAGE_FILE  \&check_message_file, $options;
+
+POST_RECEIVE \&notify_affected_refs;
+
 1;
 
 __END__
@@ -499,18 +524,60 @@ Git::Hooks::CheckJira - Git::Hooks plugin which requires citation of JIRA issues
 
 =head1 VERSION
 
-version 2.0.0
+version 3.0.0
+
+=head1 SYNOPSIS
+
+As a C<Git::Hooks> plugin you don't use this Perl module directly. Instead, you
+may configure it in a Git configuration file like this:
+
+  [githooks]
+
+    # Enable the plugin
+    plugin = CheckJira
+
+    # These users are exempt from all checks
+    admin = joe molly
+
+  [githooks "checkjira"]
+
+    # Configure the URL and the admin credentials to interact with the JIRA
+    # server.
+    jiraurl = https://jira.example.net
+    jirauser = jiradmin
+    jirapass = my-secret
+
+    # Look for JIRA keys at the beginning of the commit messages title, enclosed
+    # in brackets.
+    matchlog = (?s)^\\[([^]]+)\\]
+
+    # Impose restrictions on valid JIRA issues
+    jql = project IN (ABC, UTF, GIT) AND \
+          issuetype IN (Bug, Story) AND \
+          status IN ("In progress", "In testing")
+
+    # Require that all cited JIRA issues be assigned to the user pushing the
+    # commits.
+    by-assignee = true
+
+    # Commits pushed to master must cite JIRAs associated with the fixVersion
+    # 'future'
+    fixversion = refs/heads/master             future
+
+    # Commits pushed to release branches must cite JIRAs associated with the
+    # fixVersion named after the same major.minor version number.
+    fixversion = ^refs/heads/(\\d+\\.\\d+)\\.  ^$+
 
 =head1 DESCRIPTION
 
-This Git::Hooks plugin hooks itself to the hooks below to guarantee
-that every commit message cites at least one valid JIRA issue key in
-its log message, so that you can be certain that every change has a
-proper change request (a.k.a. ticket) open.
+This L<Git::Hooks> plugin hooks itself to the hooks below to guarantee that
+every commit message cites at least one valid JIRA issue key in its log
+message, so that you can be certain that every change has a proper change
+request (a.k.a. ticket) open.
 
 =over
 
-=item * B<commit-msg>
+=item * B<commit-msg>, B<applypatch-msg>
 
 This hook is invoked during the commit, to check if the commit message
 cites valid JIRA issues.
@@ -533,9 +600,20 @@ push>. It's used to notify JIRA of commits citing its issues via comments.
 
 =item * B<ref-update>
 
-This hook is invoked when a push request is received by Gerrit Code
+This hook is invoked when a direct push request is received by Gerrit Code
 Review, to check if the commit message cites valid JIRA issues.
 
+=item * B<commit-received>
+
+This hook is invoked when a push request is received by Gerrit Code Review to
+create a change for review, to check if the commit message cites valid JIRA
+issues.
+
+=item * B<submit>
+
+This hook is invoked when a change is submitted in Gerrit Code Review, to check
+if the commit message cites valid JIRA issues.
+
 =item * B<patchset-created>
 
 This hook is invoked when a push request is received by Gerrit Code
@@ -553,48 +631,41 @@ digits. E.g., C<CDS-123>, C<RT-1>, and C<GIT-97>.
 To enable it you should add it to the githooks.plugin configuration
 option:
 
-    git config --add githooks.plugin CheckJira
+    [githooks]
+      plugin = CheckJira
 
-=for Pod::Coverage check_codes check_commit_msg check_ref notify_commit_msg notify_ref grok_msg_jiras
+=for Pod::Coverage check_codes check_commit_msg check_ref notify_commit_msg notify_ref grok_msg_jiras check_message_file check_patchset notify_affected_refs
 
 =head1 NAME
 
-CheckJira - Git::Hooks plugin which requires citation of JIRA
-issues in commit messages.
+CheckJira - Git::Hooks plugin to implement JIRA checks
 
 =head1 CONFIGURATION
 
-The plugin is configured by the following git options.
+The plugin is configured by the following git options under the
+C<githooks.checkjira> subsection.
 
-=head2 githooks.checkjira.ref REFSPEC
+It can be disabled for specific references via the C<githooks.ref> and
+C<githooks.noref> options about which you can read in the L<Git::Hooks>
+documentation.
 
-By default, the message of every commit is checked. If you want to
-have them checked only for some refs (usually some branch under
-refs/heads/), you may specify them with one or more instances of this
-option.
-
-The refs can be specified as a complete ref name
-(e.g. "refs/heads/master") or by a regular expression starting with a
-caret (C<^>), which is kept as part of the regexp
-(e.g. "^refs/heads/(master|fix)").
-
-=head2 githooks.checkjira.jiraurl URL
+=head2 jiraurl URL
 
 This option specifies the JIRA server HTTP URL, used to construct the
 C<JIRA::REST> object which is used to interact with your JIRA
 server. Please, see the JIRA::REST documentation to know about them.
 
-=head2 githooks.checkjira.jirauser USERNAME
+=head2 jirauser USERNAME
 
 This option specifies the JIRA server username, used to construct the
 C<JIRA::REST> object.
 
-=head2 githooks.checkjira.jirapass PASSWORD
+=head2 jirapass PASSWORD
 
 This option specifies the JIRA server password, used to construct the
 C<JIRA::REST> object.
 
-=head2 githooks.checkjira.matchkey REGEXP
+=head2 matchkey REGEXP
 
 By default, JIRA keys are matched with the regex
 C</\b[A-Z][A-Z]+-\d+\b/>, meaning, a sequence of two or more capital
@@ -604,7 +675,7 @@ keys|https://confluence.atlassian.com/display/JIRA/Configuring+Project+Keys>,
 you may need to customize how this hook is going to match them. Set
 this option to a suitable regex to match a complete JIRA issue key.
 
-=head2 githooks.checkjira.matchlog REGEXP
+=head2 matchlog REGEXP
 
 By default, JIRA keys are looked for in all of the commit message. However,
 this can lead to some false positives, since the default issue pattern can
@@ -642,7 +713,7 @@ regexes are tried and JIRA keys are looked for in all of them. This allows
 you to more easily accommodate more than one way of specifying JIRA keys if
 you wish.
 
-=head2 githooks.checkjira.jql JQL
+=head2 jql JQL
 
 By default, any cited issue must exist on the server and be unresolved. You
 can specify other restrictions (and even allow for resolved issues) by
@@ -657,14 +728,20 @@ which must match all cited issues. For example, you may want to:
   [githooks "checkjira"]
     jql = resolution IS EMPTY OR resolution IS NOT EMPTY
 
-=item * Require specific projects, issuetypes, and statuses
+=item * Require specific projects, issue-types, and statuses
 
   [githooks "checkjira"]
     jql = project IN (ABC, UTF, GIT) AND issuetype IN (Bug, Story) AND status IN ("In progress", "In testing")
 
 =back
 
-=head2 githooks.checkjira.ref-jql REF JQL
+This is a scalar option. Only the last JQL expression will be used to check the
+issues.
+
+The special value 'undef' can be used to disable the option, making it possible
+to disable it locally.
+
+=head2 ref-jql REF JQL
 
 You may impose restrictions on specific branches (or, more broadly, any
 reference) by mentioning them before the JQL expression. REF can be
@@ -673,21 +750,55 @@ expression starting with a caret (C<^>), which is kept as part of the regexp
 (e.g. "^refs/heads/(master|fix)"). For instance:
 
   [githooks "checkjira"]
-    jql = refs/heads/master fixVersion = future
-    jql = ^refs/heads/release/ fixVersion IN releasedVersions()
+    ref-jql = refs/heads/master fixVersion = future
+    ref-jql = ^refs/heads/release/ fixVersion IN releasedVersions()
 
-=head2 githooks.checkjira.require [01]
+This is a scalar option. Only the last JQL expression will be used to check the
+issues.
+
+The special value 'undef' can be used to disable the option, making it possible
+to disable it locally.
+
+Note, though, that if there is a global JQL specified by the
+B<githooks.checkjira.jql> option it will be checked separately and both
+expressions must validate the issues matching REF.
+
+=head2 and-jql JQL
+
+Unlike the B<jql> option, this one is multi-valued. All JQL terms specified with
+this option are ANDed together with the last B<jql> expression and with the
+B<ref-jql> expression, if they exist.
+
+It is useful, for instance, when you want to specialize a default JQL with new
+terms.  Suppose you have a global B<jql> option with some general requirements
+that all Jira issues must fulfill, like being of particular types and in
+specific statuses:
+
+  [githooks "checkjira"]
+    jql = issuetype IN (Bug, Story) AND status IN ("In progress", "In testing")
+
+Moreover, you may have several repositories, each one associated with a
+particular Jira project. Then, you can AND more terms to the global JQL like
+this in a repository:
+
+  [githooks "checkjira"]
+    and-jql = project = ABC
+
+The special value 'undef' can be used to reset the option, effectively making
+any previous values be forgotten so that you can start anew to add terms to it.
+
+=head2 require BOOL
 
 By default, the log must reference at least one JIRA issue. You can
-make the reference optional by setting this option to 0.
+make the reference optional by setting this option to false.
 
-=head2 githooks.checkjira.unresolved [01]
+=head2 unresolved BOOL
 
 By default, every issue referenced must be unresolved, i.e., it must
 not have a resolution. You can relax this requirement by setting this
-option to 0.
+option to false.
 
-=head2 githooks.checkjira.fixversion BRANCH FIXVERSION
+=head2 fixversion BRANCH FIXVERSION
 
 This multi-valued option allows you to specify that commits affecting BRANCH
 must cite only issues that have their C<Fix For Version> field matching
@@ -723,14 +834,14 @@ the C<future> version. Also, commits affecting any branch which name begins
 with a version number (e.g. C<1.0.3>) be assigned to the corresponding JIRA
 version (e.g. C<1.0>).
 
-=head2 githooks.checkjira.by-assignee [01]
+=head2 by-assignee BOOL
 
 By default, the committer can reference any valid JIRA issue. Setting
-this value 1 requires that the user doing the push/commit (as
+this value to true requires that the user doing the push/commit (as
 specified by the C<userenv> configuration variable) be the current
 issue's assignee.
 
-=head2 githooks.checkjira.check-code CODESPEC
+=head2 check-code CODESPEC
 
 If the above checks aren't enough you can use this option to define a
 custom code to check your commits. The code may be specified directly
@@ -765,6 +876,10 @@ If you have code relying on the JIRA::Client module you're advised to
 rewrite it using the JIRA::REST module. As a stopgap measure you can
 disregard the JIRA::REST object and create your own JIRA::Client object.
 
+The routine must return a true value to signal success. It may return a false
+value or throw an exception to signal failure. It's best if it uses the 'fault'
+method to produce error messages.
+
 =item * B<ISSUES...>
 
 The remaining arguments are RemoteIssue objects representing the
@@ -772,7 +887,7 @@ issues being cited by the commit's message.
 
 =back
 
-The subroutine should return a boolean value indicating success. Any errors
+The subroutine should return a Boolean value indicating success. Any errors
 should be produced by invoking the
 B<Git::Repository::Plugin::GitHooks::error> method.
 
@@ -781,7 +896,7 @@ If the subroutine returns undef it's considered to have succeeded.
 If it raises an exception (e.g., by invoking B<die>) it's considered
 to have failed and a proper message is produced to the user.
 
-=head2 githooks.checkjira.comment VISIBILITY
+=head2 comment VISIBILITY
 
 If this option is set and the C<post-receive> hook is enabled, for every
 pushed commit, every cited JIRA issue receives a comment showing the result
@@ -812,73 +927,10 @@ In this case, the visibility isn't restricted at all.
 
 =back
 
-=head2 githooks.checkjira.skip-merges [01]
+=head2 skip-merges BOOL
 
 By default, all commits are checked. You can exempt merge commits from being
-checked by setting this option to 0.
-
-=head2 githooks.checkjira.project KEY
-
-This option is B<DEPRECATED>. Please, use a JQL expression such the
-following to restrict by project key:
-
-  project IN (ABC, GIT)
-
-By default, the committer can reference any JIRA issue in the commit
-log. You can restrict the allowed keys to a set of JIRA projects by
-specifying a JIRA project key to this option. You can allow more than one
-project by specifying this option multiple times, once per project key.
-
-If you set this option, then any cited JIRA issue that doesn't belong to one
-of the specified projects causes an error.
-
-=head2 githooks.checkjira.status STATUSNAME
-
-This option is B<DEPRECATED>. Please, use a JQL expression such the
-following to restrict by status:
-
-  status IN (Open, "In Progress")
-
-By default, it doesn't matter in which status the JIRA issues are. By
-setting this multi-valued option you can restrict the valid statuses for the
-issues.
-
-=head2 githooks.checkjira.issuetype ISSUETYPENAME
-
-This option is B<DEPRECATED>. Please, use a JQL expression such the
-following to restrict by issue type:
-
-  issuetype IN (Bug, Story)
-
-By default, it doesn't matter what type of JIRA issues are cited. By setting
-this multi-valued option you can restrict the valid issue types.
-
-=head1 EXPORTS
-
-This module exports two routines that can be used directly without
-using all of Git::Hooks infrastructure.
-
-=head2 check_affected_refs GIT
-
-This is the routine used to implement the C<update> and the
-C<pre-receive> hooks. It needs a C<Git::Repository> object.
-
-=head2 check_message_file GIT, MSGFILE
-
-This is the routine used to implement the C<commit-msg> hook. It needs
-a C<Git::Repository> object and the name of a file containing the commit
-message.
-
-=head2 check_patchset GIT, HASH
-
-This is the routine used to implement the C<patchset-created> Gerrit
-hook. It needs a C<Git::Repository> object and the hash containing the
-arguments passed to the hook by Gerrit.
-
-=head2 notify_affected_refs GIT
-
-This is the routine used to implement the C<post-receive> hook. It needs a
-C<Git::Repository> object.
+checked by setting this option to true.
 
 =head1 SEE ALSO
 
@@ -917,7 +969,7 @@ Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by CPqD <www.cpqd.com.br>.
+This software is copyright (c) 2020 by CPQD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
