@@ -1,18 +1,15 @@
-#!/usr/bin/env perl
+use warnings;
 
 package Git::Hooks::CheckRewrite;
 # ABSTRACT: Git::Hooks plugin for checking against unsafe rewrites
-$Git::Hooks::CheckRewrite::VERSION = '2.0.0'; # TRIAL
-use 5.010;
+$Git::Hooks::CheckRewrite::VERSION = '3.0.0'; # TRIAL
+use 5.016;
 use utf8;
-use strict;
-use warnings;
-use Try::Tiny;
 use Path::Tiny;
+use Log::Any '$log';
 use Git::Hooks;
 
-my $PKG = __PACKAGE__;
-(my $CFG = __PACKAGE__) =~ s/.*::/githooks./;
+my $CFG = __PACKAGE__ =~ s/.*::/githooks./r;
 
 ##########
 
@@ -36,6 +33,8 @@ sub _branches_containing {
 sub record_commit_parents {
     my ($git) = @_;
 
+    $log->debug(__PACKAGE__ . "::record_commit_parents");
+
     # Here we record the HEAD commit's own id and it's parent's ids in
     # a file under the git directory. The file has two lines in this
     # format:
@@ -58,27 +57,27 @@ sub record_commit_parents {
 sub check_commit_amend {
     my ($git) = @_;
 
+    $log->debug(__PACKAGE__ . "::check_commit_amend");
+
     my $record_file = _record_filename($git);
 
     -r $record_file
-        or $git->error($PKG, "cannot read $record_file. You probably forgot to symlink the pre-commit hook")
-            and return 0;
+        or $git->fault(<<'EOS')
+I cannot read $record_file.
+Please, check if you forgot to create the pre-commit hook.
+EOS
+        and return 0;
 
-    my ($old_commit, $old_parents) = $record_file->lines;
+    my ($old_commit, $old_parents) = $record_file->lines({chomp => 1});
 
     # For a brand new repository the commit information is empty and we
     # don't have to check anything.
     return 1 unless $old_commit;
 
-    chomp $old_commit;
     $old_commit =~ s/^commit\s+//;
 
-    if (defined $old_parents) {
-        chomp $old_parents;
-    } else {
-        # the repo's first commit has no parents.
-        $old_parents = '';
-    }
+    # the repo's first commit has no parents.
+    $old_parents //= '';
 
     my $new_parents = ($git->run(qw/rev-list --pretty=format:%P -n 1 HEAD/))[1];
 
@@ -90,18 +89,18 @@ sub check_commit_amend {
     if (@branches > 0) {
         # $old_commit is reachable by at least one branch, which means
         # the amend was unsafe.
-        my $branches = join "\n    ", @branches;
-        $git->error($PKG, "unsafe commit", <<"EOF");
-You've just performed un unsafe "git commit --amend" because your
+        my $branches = join "\n  ", @branches;
+        $git->fault(<<"EOS");
+You just performed an unsafe "git commit --amend" because your
 original HEAD ($old_commit) is still reachable by the following
 branch(es):
 
-    $branches
+  $branches
 
-Consider amending it again:
+Consider amending it again with the following command:
 
-    git commit --amend      # to amend it
-EOF
+  git commit --amend
+EOS
         return 0;
     }
 
@@ -111,28 +110,30 @@ EOF
 sub check_rebase {
     my ($git, $upstream, $branch) = @_;
 
+    $log->debug(__PACKAGE__ . "::check_rebase", {upstream => $upstream, branch => $branch});
+
     unless (defined $branch) {
         # This means we're rebasing the current branch. We try to grok
         # it's name using git symbolic-ref.
-        my $success = try {
-            chomp($branch = $git->run(qw/symbolic-ref -q HEAD/));
-        } catch {
-            # The command git symbolic-ref fails if we're in a
-            # detached HEAD. In this situation we don't care about the
-            # rewriting.
-            undef;
-        };
+        my $success = eval { $branch = $git->run(qw/symbolic-ref -q HEAD/) };
+
+        # The command git symbolic-ref fails if we're in a
+        # detached HEAD. In this situation we don't care about the
+        # rewriting.
+
         return 1 unless defined $success;
     }
 
-    # Find the base commit of the rebased sequence
-    my $base_commit = $git->run(qw/rev-list --topo-order --reverse/, "$upstream..$branch");
+    my @rebased_sequence = $git->run(qw/rev-list --topo-order --reverse/, "$upstream..$branch");
 
-    # If $upstream is a decendant of $branch, $base_commit is
+    # If $upstream is a decendant of $branch, the @rebased_sequence is
     # empty. In this situation the rebase will turn out to be a simple
     # fast-forward merge from $branch on $upstream and there is
     # nothing to lose.
-    return 1 unless $base_commit;
+    return 1 unless @rebased_sequence;
+
+    # Find the base commit of the rebased sequence
+    my $base_commit = $rebased_sequence[0];
 
     # Find all branches containing that commit
     my @branches = _branches_containing($git, $base_commit);
@@ -140,13 +141,21 @@ sub check_rebase {
     if (@branches > 1) {
         # The base commit is reachable by more than one branch, which
         # means the rewrite is unsafe.
-        my $branches = join("\n    ", grep {$_ ne $branch} @branches);
-        $git->error($PKG, "unsafe rebase", <<"EOF");
-This is an unsafe rebase because it would rewrite commits shared by
-$branch and the following other branch(es):
+        my $branches = join("\n  ", grep {$_ ne $branch} @branches);
+        $git->fault(<<"EOS");
+You just performed an unsafe rebase because it would rewrite commits shared
+by $branch and the following other branch(es):
 
-    $branches
-EOF
+  $branches
+
+If the rebase was just effected, you can reset your branch to its previous
+commit with the command:
+
+  git reset --hard \@{1}
+
+But be sure to understand the consequences of this command, as it can
+potentially make you lose work.
+EOS
         return 0;
     }
 
@@ -172,13 +181,30 @@ Git::Hooks::CheckRewrite - Git::Hooks plugin for checking against unsafe rewrite
 
 =head1 VERSION
 
-version 2.0.0
+version 3.0.0
+
+=head1 SYNOPSIS
+
+As a C<Git::Hooks> plugin you don't use this Perl module directly. Instead, you
+may configure it in a Git configuration file like this:
+
+  [githooks]
+
+    # Enable the plugin
+    plugin = CheckRewrite
+
+    # These users are exempt from all checks
+    admin = joe molly
+
+This section enables the plugin and defines the users C<joe> and C<molly> as
+administrators, effectively exempting them from any restrictions the plugin may
+impose.
 
 =head1 DESCRIPTION
 
-This Git::Hooks plugin hooks itself to the B<pre-rebase> hook to
-guarantee that it is safe in the sense that no rewritten commit is
-reachable by other branch than the one being rebased.
+This L<Git::Hooks> plugin hooks itself to the B<pre-rebase> hook to
+guarantee that it is safe in the sense that no rewritten commit is reachable
+by other branch than the one being rebased.
 
 It also hooks itself to the B<pre-commit> and the B<post-commit> hooks
 to detect unsafe B<git commit --amend> commands after the fact. An
@@ -189,7 +215,10 @@ provide a way to detect unsafe amends before committing them.
 To enable it you should add it to the githooks.plugin configuration
 option:
 
-    git config --add githooks.plugin CheckRewrite
+    [githooks]
+      plugin = CheckRewrite
+
+=for Pod::Coverage check_commit_amend check_rebase record_commit_parents
 
 =head1 NAME
 
@@ -199,38 +228,6 @@ Git::Hooks::CheckRewrite - Git::Hooks plugin for checking against unsafe rewrite
 
 There's no configuration needed or provided.
 
-=head1 EXPORTS
-
-This module exports the following routines that can be used directly
-without using all of Git::Hooks infrastructure.
-
-=head2 record_commit_parents GIT
-
-This is the routine used to implement the C<pre-commit> hook. It needs a
-C<Git::Repository> object. It simply record the original commit id and its
-parents in a file called C<GITHOOKS_CHECKREWRITE> inside the git repository
-directory
-
-=head2 check_commit_amend GIT
-
-This is the routine used to implement the C<post-commit> hook. It
-needs a C<Git::Repository> object. It reads the original commit id and its
-parents from a file called C<GITHOOKS_CHECKREWRITE> inside the git
-repository directory, which must have been created by the
-C<record_commit_parents> routine during the C<pre-commit> hook. Using
-this information it detects if this was an unsafe amend and tells the
-user so.
-
-=head2 check_rebase GIT, UPSTREAM [, BRANCH]
-
-This is the routine used to implement the C<pre-rebase> hook. It needs
-a B<Git::Repository> object, the name of the upstream branch onto which
-we're rebasing and the name of the branch being rebased. (If BRANCH is
-undefined it means that we're rebasing the current branch.
-
-The routine dies with a suitable message if it detects that it will be
-an unsafe rebase.
-
 =head1 REFERENCES
 
 Here are some references about what it means for a rewrite to be
@@ -254,7 +251,7 @@ Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by CPqD <www.cpqd.com.br>.
+This software is copyright (c) 2020 by CPQD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
