@@ -1,20 +1,17 @@
-#!/usr/bin/env perl
+use warnings;
 
 package Git::Hooks::GerritChangeId;
 # ABSTRACT: Git::Hooks plugin to insert a Change-Id in a commit message
-$Git::Hooks::GerritChangeId::VERSION = '2.0.0'; # TRIAL
-use 5.010;
+$Git::Hooks::GerritChangeId::VERSION = '3.0.0'; # TRIAL
+use 5.016;
 use utf8;
-use strict;
-use warnings;
+use Carp;
+use Log::Any '$log';
 use Git::Hooks;
 use Git::Message;
 use Path::Tiny;
-use Carp;
-use Try::Tiny;
 
-my $PKG = __PACKAGE__;
-(my $CFG = __PACKAGE__) =~ s/.*::/githooks./;
+my $CFG = __PACKAGE__ =~ s/.*::/githooks./r;
 
 ##########
 
@@ -23,7 +20,8 @@ sub gen_change_id {
 
     my $filename = Path::Tiny->tempfile(UNLINK => 1);
     open my $fh, '>', $filename ## no critic (RequireBriefOpen)
-        or croak "$PKG: internal error: can't open $filename for writing: $!";
+        or $git->fault("Internal error: can't open '$filename' for writing:", {details => $!})
+        and croak;
 
     foreach my $info (
         [ tree      => [qw/write-tree/] ],
@@ -31,10 +29,11 @@ sub gen_change_id {
         [ author    => [qw/var GIT_AUTHOR_IDENT/] ],
         [ committer => [qw/var GIT_COMMITTER_IDENT/] ],
     ) {
-        # It's OK if we can't find info.
-        try {
-            $fh->print($info->[0], ' ', scalar($git->run(@{$info->[1]})));
-        };
+        my $value = eval { $git->run(@{$info->[1]}) };
+        if (defined $value) {
+            # It's OK if we can't find value.
+            $fh->print("$info->[0] $value");
+        }
     }
 
     $fh->print("\n", $msg);
@@ -69,9 +68,12 @@ sub insert_change_id {
 sub rewrite_message {
     my ($git, $commit_msg_file) = @_;
 
+    $log->debug(__PACKAGE__ . "::rewrite_message($commit_msg_file)");
+
     my $msg = eval { $git->read_commit_msg_file($commit_msg_file) };
     unless (defined $msg) {
-        $git->error($PKG, "cannot read commit message file '$commit_msg_file'", $@);
+        $git->fault("Internal error: cannot read commit message file '$commit_msg_file'",
+                    {details => $@});
         return 0;
     }
 
@@ -84,7 +86,8 @@ sub rewrite_message {
 }
 
 # Install hooks
-COMMIT_MSG \&rewrite_message;
+APPLYPATCH_MSG \&rewrite_message;
+COMMIT_MSG     \&rewrite_message;
 
 1;
 
@@ -100,15 +103,32 @@ Git::Hooks::GerritChangeId - Git::Hooks plugin to insert a Change-Id in a commit
 
 =head1 VERSION
 
-version 2.0.0
+version 3.0.0
+
+=head1 SYNOPSIS
+
+As a C<Git::Hooks> plugin you don't use this Perl module directly. Instead, you
+may configure it in a Git configuration file like this:
+
+  [githooks]
+
+    # Enable the plugin
+    plugin = CheckGerritChangeId
+
+    # These users are exempt from all checks
+    admin = joe molly
+
+The first section enables the plugin and defines the users C<joe> and C<molly>
+as administrators, effectively exempting them from any restrictions the plugin
+may impose.
 
 =head1 DESCRIPTION
 
-This Git::Hooks plugin hooks itself to the C<commit-msg> hook. It is a
-reimplementation of Gerrit's official commit-msg hook for inserting
-change-ids in git commit messages.  It's does not produce the same
-C<Change-Id> for the same message, but this is not really necessary,
-since it keeps existing Change-Id footers unmodified.
+This L<Git::Hooks> plugin hooks itself to the C<commit-msg> and the
+C<applypatch-msg> hooks. It is a reimplementation of Gerrit's official
+commit-msg hook for inserting change-ids in git commit messages.  It's does not
+produce the same C<Change-Id> for the same message, but this is not really
+necessary, since it keeps existing Change-Id footers unmodified.
 
 (What follows is a partial copy of that document's DESCRIPTION
 section.)
@@ -152,29 +172,20 @@ existing change viewed on the web.
 To enable the plugin you should add it to the githooks.plugin
 configuration option:
 
-    git config --add githooks.plugin GerritChangeId
+    [githooks]
+      plugin = GerritChangeId
 
-=for Pod::Coverage gen_change_id insert_change_id
+=for Pod::Coverage gen_change_id insert_change_id rewrite_message
 
 =head1 NAME
 
-Git::Hooks::GerritChangeId - Git::Hooks plugin to insert a Change-Id in a commit message.
+Git::Hooks::GerritChangeId - Git::Hooks plugin to insert Change-Ids in
+commit messages
 
 =head1 CONFIGURATION
 
 There's no configuration needed or provided.
 
-=head1 EXPORTS
-
-This module exports one routine that can be used directly without
-using all of Git::Hooks infrastructure.
-
-=head2 rewrite_message GIT, MSGFILE
-
-This is the routine used to implement the C<commit-msg> hook. It needs a
-C<Git::Repository> object and the name of a file containing the commit
-message.
-
 =head1 REFERENCES
 
 =over
@@ -195,7 +206,7 @@ Gustavo L. de M. Chaves <gnustavo@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2017 by CPqD <www.cpqd.com.br>.
+This software is copyright (c) 2020 by CPQD <www.cpqd.com.br>.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
